#ifndef CCP210X_H
#define CCP210X_H

// {FA078453-723A-45F0-92A6-C156B4B3568B}
DEFINE_GUID(GUID_BAUDCONFIG,
    0xFA078453, 0x723A, 0x45F0, 0x92, 0xA6, 0xC1, 0x56, 0xB4, 0xB3, 0x56, 0x8B);

class CCP210x : public ICP210x, public IProvideClassInfo, public ISupportErrorInfo
{
protected:
    inline IUnknown* GetUnknown() noexcept { return(static_cast<ICP210x*>(this)); }
    inline HRESULT RaiseCOMException(DISPID dispid, HRESULT hr) noexcept { return(::RaiseCOMException(this, dispid, hr)); }

public:
    // IUnknown
    IFACEMETHOD(QueryInterface) (REFIID riid, void** ppvObject);
    IFACEMETHOD_(ULONG, AddRef) ();
    IFACEMETHOD_(ULONG, Release)();

    // IDispatch
    IFACEMETHOD(GetTypeInfoCount)(UINT* pctinfo);
    IFACEMETHOD(GetTypeInfo)(UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo);
    IFACEMETHOD(GetIDsOfNames)(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId);
    IFACEMETHOD(Invoke)(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr);

    // ICP210x
    IFACEMETHOD(get_Devices)(IDispatch** ppDevices);
    IFACEMETHOD(GetPartNumber)(enum PartNumbers* pPartNum);
    IFACEMETHOD(SetVid)(long VID);
    IFACEMETHOD(SetPid)(long PID);
    IFACEMETHOD(SetProductString)(BSTR strProduct);
    IFACEMETHOD(SetSerialNumber)(BSTR strSerialNumber);
    IFACEMETHOD(SetSelfPower)(VARIANT_BOOL SelfPower);
    IFACEMETHOD(SetMaxPower)(long MaxPower);
    IFACEMETHOD(SetDeviceVersion)(long lVersion);
    IFACEMETHOD(SetLockValue)();
    IFACEMETHOD(GetDeviceProductString)(BSTR* ProductString);
    IFACEMETHOD(GetDeviceSerialNumber)(BSTR* SerNum);
    IFACEMETHOD(GetDeviceVid)(long* VID);
    IFACEMETHOD(GetDevicePid)(long* PID);
    IFACEMETHOD(GetSelfPower)(VARIANT_BOOL* SelfPower);
    IFACEMETHOD(GetMaxPower)(long* MaxPower);
    IFACEMETHOD(GetDeviceVersion)(long* wVersion);
    IFACEMETHOD(GetLockValue)(VARIANT_BOOL* LockVal);
    IFACEMETHOD(Reset)();
    IFACEMETHOD(CreateHexFile)(BSTR bstrFileName);
    IFACEMETHOD(GetBaudRateConfig)(SAFEARRAY** ppBaudConfig);
    IFACEMETHOD(SetBaudRateConfig)(SAFEARRAY** ppBaudConfig);
    IFACEMETHOD(GetPortConfig)(PortConfig* pConfig);
    IFACEMETHOD(SetPortConfig)(PortConfig* pConfig);
    IFACEMETHOD(GetFlushBufferConfig)(long* pbFlushBufferConfig);
    IFACEMETHOD(SetFlushBufferConfig)(long bFlushBufferConfig);
    IFACEMETHOD(SetInterfaceString)(long InterfaceNumber, BSTR strInterface);
    IFACEMETHOD(GetDeviceInterfaceString)(long InterfaceNumber, BSTR* strInterface);
    IFACEMETHOD(SetManufacturerString)(THIS_ BSTR strManufacturer);
    IFACEMETHOD(GetDeviceManufacturerString)(THIS_ BSTR* strManufacturer);
    IFACEMETHOD(SetDeviceMode)(enum DeviceMode ECI, enum DeviceMode SCI);
    IFACEMETHOD(GetDeviceMode)(enum DeviceMode* ECI, enum DeviceMode* SCI);

    // IProvideClassInfo
    IFACEMETHOD(GetClassInfo)(ITypeInfo** ppTI);

    // ISupportErrorInfo
    IFACEMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

    CCP210x(ICP210xDevices* pDevices, DWORD dwDeviceNum, HRESULT* phr);
    virtual ~CCP210x();

private:
    ICP210xDevices* m_pDevices;
    ITypeInfo*      m_pTypeInfo;
    HANDLE          m_hDevice;
    volatile ULONG  m_lRefCount;
#ifdef _WIN64
    BYTE bPadding[4]; // C4820
#endif
};

#endif
