#include "precomp.h"

CClassFactory::CClassFactory() noexcept
    : m_lRefCount(1)
{
    ::LockServer();
}

CClassFactory::~CClassFactory()
{
    ::UnlockServer();
}

IFACEMETHODIMP
CClassFactory::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IClassFactory))
            *ppvObject = static_cast<IClassFactory*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CClassFactory::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CClassFactory::Release()
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CClassFactory::LockServer(BOOL fLock)
{
    if(fLock)
        ::LockServer();
    else
        ::UnlockServer();

    return(S_OK);
}
