#include "precomp.h"

HRESULT StatusToHresult(CP210x_STATUS stat)
{
    static const struct
    {
        CP210x_STATUS   stat;
        HRESULT         hr;
    } Map[] =
    {
        { CP210x_SUCCESS,                   S_OK                                        },
        { CP210x_INVALID_HANDLE,            E_HANDLE                                    },
        { CP210x_INVALID_PARAMETER,         E_INVALIDARG                                },
        { CP210x_FUNCTION_NOT_SUPPORTED,    __HRESULT_FROM_WIN32(ERROR_NOT_SUPPORTED)   },
        { CP210x_DEVICE_IO_FAILED,          __HRESULT_FROM_WIN32(ERROR_IO_DEVICE)       }
    };

    for(size_t x = 0; x < _countof(Map); ++x)
        if(Map[x].stat == stat)
            return(Map[x].hr);

    return(MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, (stat + 0x0200)));
}

HRESULT BSTRFromStringA(const char* pstr, BSTR* pbstr) noexcept
{
    const int nLen = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, pstr, -1, nullptr, 0);

    if(!nLen)
    {
        *pbstr = nullptr;
        return(HRESULT_FROM_WIN32(GetLastError()));
    }

    if(nullptr != (*pbstr = SysAllocStringLen(nullptr, static_cast<UINT>(nLen) - 1U)))
    {
        if(MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, pstr, -1, *pbstr, nLen))
            return(S_OK);

        const HRESULT hr = HRESULT_FROM_WIN32(GetLastError());
        SysFreeString(*pbstr);
        *pbstr = nullptr;
        return(hr);
    }

    return(E_OUTOFMEMORY);
}
